import { ComponentFixture, TestBed } from '@angular/core/testing';

import { GalleryComponent } from './gallery.component';
import { provideRouter } from '@angular/router';
import { GalleryService } from '../../../services/gallery.service';
import { provideHttpClient } from '@angular/common/http';
import { provideHttpClientTesting } from '@angular/common/http/testing';
import { of } from 'rxjs';
import { ImageType } from '../../../models/image.type';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';


describe(GalleryComponent.name, () => {

  let component: GalleryComponent;
  let fixture: ComponentFixture<GalleryComponent>;
  let native: HTMLElement;
  let galleryService: jasmine.SpyObj<GalleryService>;

  const weddingId = '4d35e8f0-b155-4e79-b5e8-f0b1550e7940';
  const guestId = 'e501bf41-d3d2-417d-81bf-41d3d2217d0a';

  beforeEach(async () => {
    const spy = jasmine.createSpyObj<GalleryService>('GalleryService', ['getGalleryByWeddingId', 'postImage', 'changeImageLikes']);

    await TestBed.configureTestingModule({
      imports: [
        GalleryComponent,
        NoopAnimationsModule
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: GalleryService,
          useValue: spy
        }
      ]
    })
      .compileComponents();

    fixture = TestBed.createComponent(GalleryComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    component.guestId = guestId;
    component.weddingId = weddingId;
    galleryService = TestBed.inject(GalleryService) as jasmine.SpyObj<GalleryService>;

    const gallery :  [ImageType] = [
      {
        id: '84621198-9a7a-425a-a211-989a7a525a23',
        date: '2024-11-12 12:12',
        liked: false,
        likes: 0
      }
    ];
    galleryService.getGalleryByWeddingId.and.returnValue(of(gallery));

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render gallery', () => {
    let containers = native.querySelectorAll('.save-image-container');
    expect(containers.length).toBe(1);
  });

  it('should call onDownloadImageAction when download button is clicked', () => {
    spyOn(component, 'onDownloadImageAction');
    fixture.detectChanges();

    const downloadButton = native.querySelectorAll('.save-image-container p-button button');
    downloadButton.item(0).dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.onDownloadImageAction).toHaveBeenCalledWith('84621198-9a7a-425a-a211-989a7a525a23');
  });

  it('should show upload dialog for upload when add image button is clicked', () => {
    fixture.detectChanges();
    let addButton= native.querySelector('.add-image-container p-button button');

    addButton!.dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.displayDialog).toBe(true);
  });

  it('should upload image when onUpload is called', () => {
    galleryService.postImage.and.returnValue(of(null));

    const fakeFile = new File([''], 'test.jpg', { type: 'image/jpeg' });
    const event = {
      files: [fakeFile]
    };

    component.onUpload(event);

    expect(galleryService.postImage).toHaveBeenCalledWith(fakeFile, weddingId, guestId);
  });

});

