import {Component, OnInit, signal, WritableSignal} from '@angular/core';
import {ActivatedRoute, Router, RouterLink} from '@angular/router';
import { QuizService } from '../../../services/quiz.service';
import {Button} from 'primeng/button';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {catchError} from 'rxjs';
import {QuizStateType} from '../../../models/quizState.type';
import {NgIf} from '@angular/common';
import {WhoKnowsBetterQuizStateType} from '../../../models/whoKnowsBetterQuizState.type';
import { WhoKnowsBetterQuizService } from '../../../services/who-knows-better-quiz.service';

@Component({
  selector: 'app-games-list',
  standalone: true,
  imports: [
    RouterLink,
    Button,
    ReactiveFormsModule,
    FormsModule,
    NgIf
  ],
  templateUrl: './games-list.component.html',
  styleUrl: './games-list.component.css'
})
export class GamesListComponent implements OnInit {
  constructor(private router: Router,private route: ActivatedRoute,private QuizService: QuizService, private whoKnowsBetterQuizService: WhoKnowsBetterQuizService) {}

  weddingId: WritableSignal<string | null> = signal<string | null>(null);
  guestId: WritableSignal<string | null> = signal<string | null>(null);
  weddingIdAsString: string | null = this.weddingId();
  guestIdAsString: string | null = this.guestId();

  quizCompleted: QuizStateType= {quizFinished:false};
  whoKnowsBetterCompleted: WhoKnowsBetterQuizStateType= {whoKnowsQuizFinished:false};

  RootToGame(cardName: string) {
    if(cardName=='Quiz'){
      this.router.navigate(['/wedding', this.weddingId(),this.guestId() , 'games','quiz']);
    }
    else if (cardName === 'who-knows-better'){
      this.router.navigate(['/wedding', this.weddingId() , this.guestId(), 'games', 'who-knows-better']);
    }
    else if (cardName === 'Wyniki'){
      this.router.navigate(['/wedding', this.weddingId(),this.guestId() , 'games','hall']);
    }
    else if (cardName === 'Podsumowanie'){
      this.router.navigate(['/wedding', this.weddingId(),this.guestId() , 'games','who-knows-better', 'results']);
    }
  }

  ngOnInit(): void {
    this.route.paramMap.subscribe((params) => {
      this.weddingId.set(params.get('weddingId'));
      this.guestId.set(params.get('guestId'));
    });
    this.getState();
    this.getWhoKnowsState();
  }


  arrowClick(direction: 'left' | 'right'): void {
    const carrousel = document.querySelector('.carrousel') as HTMLElement;

    if (!carrousel) {
      console.error('Carrousel element not found');
      return;
    }

    const scrollAmount = carrousel.clientWidth; // Przewijanie o szerokość jednego widoku (pełnej karty)

    if (direction === 'left') {
      carrousel.scrollBy({ left: -scrollAmount, behavior: 'smooth' });
    } else if (direction === 'right') {
      carrousel.scrollBy({ left: scrollAmount, behavior: 'smooth' });
    }
  }

  getState(): void {
    const weddingId: string = this.weddingId() ?? '';
    const guestId: string = this.guestId() ?? '';
    this.QuizService
      .getWeddingQuizState(weddingId,guestId)
      .pipe(catchError((error) => {
        console.error(error);
        throw error;
      }))
      .subscribe((weddingQuizState) => {
        this.quizCompleted = weddingQuizState;
      });
  }

  getWhoKnowsState(): void {
    const weddingId: string = this.weddingId() ?? '';
    const guestId: string = this.guestId() ?? '';
    this.whoKnowsBetterQuizService
      .getWhoKnowsBetterQuizStateByWeddingId(weddingId, guestId)
      .pipe(catchError((error) => {
        console.error(error);
        throw error;
      }))
      .subscribe((whoKnowsBetterQuizState) => {
        this.whoKnowsBetterCompleted = whoKnowsBetterQuizState;
      });
  }

}
