import {Component, Input, OnInit} from '@angular/core';
import { HallOfFameService } from '../../../services/hall-of-fame.service';
import {ActivatedRoute, Router, RouterLink} from '@angular/router';
import {TableModule} from 'primeng/table';
import {catchError} from 'rxjs';
import {HallOfFameType} from '../../../models/hallOfFame.type';
import {QuizType} from '../../../models/quiz.type';
import {NgClass, NgForOf} from '@angular/common';

@Component({
  selector: 'app-hall-of-fame',
  standalone: true,
  imports: [
    TableModule,
    NgClass,
  ],
  templateUrl: './hall-of-fame.component.html',
  styleUrl: './hall-of-fame.component.css',
  host: { ngSkipHydration: 'true' }
})
export class HallOfFameComponent implements OnInit {

  weddingScores: HallOfFameType[]=[];

  @Input()
  weddingId: string = '';

  @Input()
  guestId: string = '';

  tableHeight: string='';

  constructor(private route: ActivatedRoute,private HallOfFameService: HallOfFameService) {}

  ngOnInit() {
    if (typeof window !== 'undefined'){
      const windowHeight = window.innerHeight;
      this.tableHeight = `${windowHeight - 66}px`;
    }

    if (this.weddingId) {
      this.loadResults();
    }
  }

  loadResults() {
    this.HallOfFameService
      .getScoreByWeddingId(this.weddingId,this.guestId)
      .pipe(catchError((error) => {
        console.error('Error fetching quiz scores:', error);
        throw error;
      }))
      .subscribe((weddingScores) => {
        this.weddingScores = this.sortScores(weddingScores);
      });
  }

  sortScores(scoreType: HallOfFameType[]): HallOfFameType[]{
    scoreType.sort((a, b) => b.quizPoints - a.quizPoints);
    return scoreType;
  }

}
