import { ComponentFixture, TestBed } from '@angular/core/testing';

import { PresentGuestComponent } from './present-guest.component';
import {PresentService} from '../../../services/present.service';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {provideRouter} from '@angular/router';
import {provideHttpClient} from '@angular/common/http';
import {provideHttpClientTesting} from '@angular/common/http/testing';
import {PresentType} from '../../../models/presents.type';
import {of} from 'rxjs';
import {Confirmation} from 'primeng/api';

describe(PresentGuestComponent.name, () => {
  let component: PresentGuestComponent;
  let fixture: ComponentFixture<PresentGuestComponent>;
  let native: HTMLElement;
  let presentService:jasmine.SpyObj<PresentService>;

  const weddingId = '4d35e8f0-b155-4e79-b5e8-f0b1550e7940';
  const guestId = 'e501bf41-d3d2-417d-81bf-41d3d2217d0a';

  beforeEach(async () => {
    const spy = jasmine.createSpyObj<PresentService>('PresentService',['getPresentsByWeddingIdGuestSide','sendPresentsToEditByWeddingId']);

    await TestBed.configureTestingModule({
      imports: [
        PresentGuestComponent,
        NoopAnimationsModule
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: PresentService,
          useValue: spy
        }
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(PresentGuestComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    component.guestId = guestId;
    component.weddingId = weddingId;
    presentService = TestBed.inject(PresentService) as jasmine.SpyObj<PresentService>;

    const present: [PresentType]=[
      {
        id: '84621198-9a7a-425a-a211-989a7a525a23',
        name: 'iksboks',
        linkToPresent: 'best/site/with/iksbokses',
        description: 'iksbox 360',
        reservedBy: '84621198-9a7a-425a-a211-989a7a525a21',
      }
    ]
    presentService.getPresentsByWeddingIdGuestSide.and.returnValue(of(present));

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render presents view', () => {
    let containers = native.querySelectorAll('.text-lg.font-medium.text-900.mt-2');
    expect(containers.length).toBe(1);
  });

  it('should send present when reservePresent is called with present that exists', () => {
    presentService.sendPresentsToEditByWeddingId.and.returnValue(of(null));

    spyOn(component['confirmationService'], 'confirm').and.callFake((confirmation: Confirmation) => { return confirmation.accept?.(); });

    const present={
      id: '84621198-9a7a-425a-a211-989a7a525a23',
      name: 'iksboks',
      linkToPresent: 'best/site/with/iksbokses',
      description: 'iksbox 360',
      reservedBy: 'e501bf41-d3d2-417d-81bf-41d3d2217d0a',
    }
    component.guestId = guestId;
    component.weddingId = weddingId;

    fixture.detectChanges();

    component.reservePresent(present);

    expect(presentService.sendPresentsToEditByWeddingId).toHaveBeenCalledWith(weddingId,guestId,present);
  });

});
