import {Component, Input, OnInit} from '@angular/core';
import {Button} from 'primeng/button';
import {DataViewModule} from 'primeng/dataview';
import {DialogModule} from 'primeng/dialog';
import {InputTextareaModule} from 'primeng/inputtextarea';
import {NgClass, NgForOf, NgIf} from '@angular/common';
import {ConfirmationService, MessageService, PrimeTemplate} from 'primeng/api';
import {ReactiveFormsModule} from '@angular/forms';
import {TagModule} from 'primeng/tag';
import {PresentType} from '../../../models/presents.type';
import {ActivatedRoute, Router} from '@angular/router';
import {PresentService} from '../../../services/present.service';
import {ToastModule} from 'primeng/toast';
import {ConfirmDialogModule} from 'primeng/confirmdialog';

@Component({
  selector: 'app-present-guest',
  standalone: true,
  imports: [
    Button,
    DataViewModule,
    DialogModule,
    InputTextareaModule,
    NgForOf,
    NgIf,
    PrimeTemplate,
    ReactiveFormsModule,
    TagModule,
    NgClass,
    ToastModule,
    ConfirmDialogModule
  ],
  providers: [MessageService, ConfirmationService],
  templateUrl: './present-guest.component.html',
  styleUrl: './present-guest.component.css',
  host: { ngSkipHydration: 'true' }
})
export class PresentGuestComponent implements OnInit {

  constructor(private router: Router,
              private route: ActivatedRoute,
              private PresentService: PresentService,
              private confirmationService: ConfirmationService,
              private messageService: MessageService,
  ) {}

  weddingPresents: PresentType[] =[];

  @Input()
  weddingId: string = '';

  @Input()
  guestId: string = '';

  ngOnInit(): void {
    this.getPresents();
  }

  getPresents(): void {
    this.PresentService
      .getPresentsByWeddingIdGuestSide(this.weddingId, this.guestId)
      .subscribe({
        next: (weddingPresents) => {
          this.weddingPresents = weddingPresents;
        },
        error: (error) => console.error('Error fetching presents:', error)
      });
  }

  reservePresent(present: PresentType) {
    this.confirmationService.confirm({
      header: 'Rezerwacja',
      message: 'Zobowiązujesz się do zakupu tego prezentu, czy jesteś pewny?',
      acceptLabel: 'Tak',
      rejectLabel: 'Nie',
      acceptIcon: 'pi pi-check mr-2',
      rejectIcon: 'pi pi-times mr-2',
      rejectButtonStyleClass: 'p-button-sm',
      acceptButtonStyleClass: 'p-button-outlined p-button-sm',
      accept: () => {
        const presentBody = { ...present };
        presentBody.reservedBy=this.guestId;
        this.PresentService.sendPresentsToEditByWeddingId(this.weddingId, this.guestId, presentBody).subscribe({
          next: () => {
            const index = this.weddingPresents.findIndex(p => p.id === presentBody.id);
            this.weddingPresents[index] = {... presentBody};
            this.weddingPresents = [...this.weddingPresents];

            this.messageService.add({ severity: 'info', summary: 'Zarezerwowano', detail: 'Prezent zarezerwowany', life: 3000 });
          }
        });
      }
    });
  }

  cancelPresent(present: PresentType) {
    this.confirmationService.confirm({
      header: 'Rezygnacja',
      message: 'Rezygnujesz z zakupu tego prezentu?',
      acceptLabel: 'Tak',
      rejectLabel: 'Nie',
      acceptIcon: 'pi pi-check mr-2',
      rejectIcon: 'pi pi-times mr-2',
      rejectButtonStyleClass: 'p-button-sm',
      acceptButtonStyleClass: 'p-button-outlined p-button-sm',
      accept: () => {
        const presentBody = { ...present };
        presentBody.reservedBy='';
        this.PresentService.sendPresentsToEditByWeddingId(this.weddingId, this.guestId, presentBody).subscribe({
          next: () => {
            const index = this.weddingPresents.findIndex(p => p.id === presentBody.id);
            presentBody.reservedBy=null;
            this.weddingPresents[index] = {... presentBody};
            this.weddingPresents = [...this.weddingPresents];

            this.messageService.add({ severity: 'info', summary: 'Zrezygnowano', detail: 'Juz nie rezerwujesz tego prezentu', life: 3000 });
          }
        });
      }
    });
  }

  checkPresentReservation(present: PresentType) {
    if(present.reservedBy == this.guestId) {
      return "Twoja rezerwacja"
    }else if(present.reservedBy != null) {
      return "Zarezerwowane"
    }else {
      return "Dostępne"
    }
  }
}
