import {ComponentFixture, TestBed} from '@angular/core/testing';
import {QuizComponent} from './quiz.component';
import {QuizService} from '../../../services/quiz.service';
import {provideRouter, Router} from '@angular/router';
import {of} from 'rxjs';
import {provideHttpClient} from '@angular/common/http';
import {provideHttpClientTesting} from '@angular/common/http/testing';
import {QuizType} from '../../../models/quiz.type';
import {QuizScoreType} from '../../../models/quizScore.type';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';

describe(QuizComponent.name, () => {

  let component: QuizComponent;
  let fixture: ComponentFixture<QuizComponent>;
  let router: Router;
  let native: HTMLElement;
  let quizService: jasmine.SpyObj<QuizService>;

  const weddingId = '4d35e8f0-b155-4e79-b5e8-f0b1550e7940';
  const guestId = 'e501bf41-d3d2-417d-81bf-41d3d2217d0a';

  beforeEach(async() => {
    const spy=jasmine.createSpyObj<QuizService>('QuizService',['getWeddingQuizQuestion','sendGuestPointsToUpdate','getWeddingQuizState','getWeddingQuizScoreOfGuest']);

    await TestBed.configureTestingModule({
      imports: [
        QuizComponent,
        NoopAnimationsModule
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: QuizService,
          useValue: spy
        }
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(QuizComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    component.guestId= guestId;
    component.weddingId = weddingId;
    router = TestBed.inject(Router);
    quizService = TestBed.inject(QuizService) as jasmine.SpyObj<QuizService>;

    const question : QuizType= {
        question: 'What is 2 + 2?',
        answer1: '4',
        answer2: '3',
        answer3: '5',
        answer4: '21',
    };
    quizService.getWeddingQuizQuestion.and.returnValue(of(question));
    component.shuffledAnswers = [question.answer2, question.answer1, question.answer4, question.answer3];

    const quizState: QuizScoreType = {
      questionIndex: 0,
      quizPoints:0,
      guestName:'',
    };
    quizService.getWeddingQuizScoreOfGuest.and.returnValue(of(quizState));

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render quiz', () => {
    let containers = native.querySelectorAll('.flex.justify-content-end.w-screen.h-screen');
    expect(containers.length).toBe(1);
  });

  it('should call onAnswerSelected when the first answer button is clicked', () => {
    spyOn(component,'onAnswerSelected')
    fixture.detectChanges();
    const answerButtons = native.querySelectorAll('.grid button');

    answerButtons.item(0).dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.onAnswerSelected).toHaveBeenCalledWith(component.shuffledAnswers[0]);
  });

  it('should call getQuestion when the next question button is clicked', () => {
    spyOn(component,'getQuestion')
    component.selectedAnswer=component.shuffledAnswers[0];
    fixture.detectChanges();
    const answerButtons = native.querySelectorAll('.flex p-button');

    answerButtons.item(0).dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.getQuestion).toHaveBeenCalled();
  });

  it('should call onClickRoute when the redirect to hall is clicked', () => {
    const navigateSpy = spyOn(router, 'navigateByUrl');
    component.weddingId=weddingId;
    component.guestId=guestId;
    component.quizCompleted.quizFinished=true;
    fixture.detectChanges();
    const nextButton = native.querySelector('.text-center p-button');

    nextButton!.dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(navigateSpy).toHaveBeenCalledWith('/wedding/4d35e8f0-b155-4e79-b5e8-f0b1550e7940/e501bf41-d3d2-417d-81bf-41d3d2217d0a/games/hall');
  });

});
