import { ComponentFixture, TestBed } from '@angular/core/testing';

import { WeddingComponent } from './wedding.component';
import { WeddingService } from '../../../services/wedding.service';
import { provideRouter } from '@angular/router';
import { provideHttpClient } from '@angular/common/http';
import { provideHttpClientTesting } from '@angular/common/http/testing';
import { of } from 'rxjs';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { WeddingType } from '../../../models/wedding.type';
import { SharedService } from '../../../services/shared.service';
import {MockSecureImagePipe as SecuredImagePipe} from '../../../pipes/mock-secure-image.pipe';
import {By} from '@angular/platform-browser';

describe('WeddingComponent', () => {
  let component: WeddingComponent;
  let fixture: ComponentFixture<WeddingComponent>;
  let native: HTMLElement;
  let weddingService: jasmine.SpyObj<WeddingService>;
  let sharedService: jasmine.SpyObj<SharedService>;

  const weddingId = '4d35e8f0-b155-4e79-b5e8-f0b1550e7940';

  beforeEach(async () => {
    const spyWeddingService = jasmine.createSpyObj<WeddingService>('WeddingService', ['getWeddingById', 'getWeddingBackground']);
    const spyShardedService = jasmine.createSpyObj<SharedService>('SharedService', ['changeWeddingMessage']);

    await TestBed.configureTestingModule({
      imports: [
        WeddingComponent,
        SecuredImagePipe,
        NoopAnimationsModule
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        { provide: WeddingService, useValue: spyWeddingService },
        { provide: SharedService, useValue: spyShardedService }
      ]
    })
      .compileComponents();

    fixture = TestBed.createComponent(WeddingComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    component.weddingId = weddingId;
    weddingService = TestBed.inject(WeddingService) as jasmine.SpyObj<WeddingService>;
    sharedService = TestBed.inject(SharedService) as jasmine.SpyObj<SharedService>;

    const wedding : WeddingType =
    {
      id: weddingId,
      name: 'Test wedding',
      eventStartDateTime: '2020-20-20 20:20'
    };
    weddingService.getWeddingById.and.returnValue(of(wedding));
    sharedService.changeWeddingMessage.and.returnValue();

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  // it('should render background image with correct src', () => {
  //   const imgElement = native.querySelector('#background-img') as HTMLImageElement;
  //
  //   expect(imgElement).toBeTruthy();
  //   expect(imgElement.getAttribute('src')).toContain('blob:nodedata:4172ae66-d889-4030-b01f-af305e911392');
  // });

  it('should move background image when carrousel scroll action is triggered', () => {
    spyOn(component, 'moveBackgroundImg');

    const pCarousel = fixture.debugElement.query(By.css('p-carousel'));
    pCarousel.triggerEventHandler('onPage', { page: 1 });

    expect(component.moveBackgroundImg).toHaveBeenCalled();
  });

});
