import {
  Component,
  OnInit,
  Input, ViewChild, ElementRef,
  Injector,
  InjectionToken,
} from '@angular/core';
import {WeddingService} from '../../../services/wedding.service';
import {WeddingType} from '../../../models/wedding.type';
import {SliderModule} from 'primeng/slider';
import {CommonModule} from '@angular/common';
import {CarouselModule} from 'primeng/carousel';
import {WelcomeCardComponent} from '../carousel-components/welcome-card/welcome-card.component';
import {InfoCardComponent} from '../carousel-components/info-card/info-card.component';
import { SecuredImagePipe } from '../../../pipes/secured-image.pipe';
import {SharedService} from '../../../services/shared.service';
import { InvitationCardComponent } from '../carousel-components/invitation-card/invitation-card.component';

@Component({
  selector: 'app-wedding',
  standalone: true,
  imports: [
    SliderModule,
    CommonModule,
    CarouselModule,
    SecuredImagePipe
  ],
  templateUrl: './wedding.component.html',
  styleUrl: './wedding.component.css'
})
export class WeddingComponent implements OnInit {

  @Input()
  weddingId!: string;

  wedding: WeddingType | null = null;

  carouselItems = [
    { component: InvitationCardComponent, data: () => ({ wedding: this.wedding }) },
    { component: InfoCardComponent, data: () => ({ wedding: this.wedding }) }
  ];

  constructor(
    private weddingService : WeddingService,
    private sharedService: SharedService,
    private injector: Injector
  ) {}

  createInjector(item: any): Injector {
    return Injector.create({
      providers: [
        {
          provide: 'componentData',
          useValue: { wedding: this.wedding }
        }
      ],
      parent: this.injector
    });
  }

  ngOnInit(): void {
    this.weddingService.getWeddingById(this.weddingId).subscribe({
      next: (wedding) => {
        this.wedding = wedding;

        let weddingDate = new Date(wedding.eventStartDateTime);
        let today = new Date();

        this.sharedService.changeWeddingMessage(weddingDate.getTime() - today.getTime() < 86400000); // 86400000 = 24h
      },
      error: (error) => console.error('Error getting wedding:', error)
    })
  }

  @ViewChild('bi', { static: true }) backgroundImage!: ElementRef<HTMLImageElement>;

  onCarrouselScroll(event: any) {
    if (this.backgroundImage) {
      this.moveBackgroundImg((window.innerWidth - this.backgroundImage.nativeElement.clientWidth) / (this.carouselItems.length - 1) * event.page)
    }
  }

  moveBackgroundImg(amount: number): void {
    this.backgroundImage.nativeElement.animate(
      [
        { transform: `translateX(${amount}px)` }
      ],
      {
        duration: 2000,
        easing: 'ease-in-out',
        fill: 'forwards'
      }
    );
  }

}
