import { ComponentFixture, TestBed } from '@angular/core/testing';
import { WhoKnowsBetterQuizComponent } from './who-knows-better-quiz.component';
import {of} from 'rxjs';
import {WhoKnowsBetterQuizService} from '../../../services/who-knows-better-quiz.service';
import {ActivatedRoute, Router} from '@angular/router';

describe('WhoKnowsBetterQuizComponent', () => {
  let component: WhoKnowsBetterQuizComponent;

  const mockWhoKnowsBetterQuizService = {
    getWhoKnowsBetterQuizStateByWeddingId: jasmine.createSpy('getWhoKnowsBetterQuizStateByWeddingId')
      .and.returnValue(of({ whoKnowsQuizFinished: false })),
    getQuestion: jasmine.createSpy('getQuestion').and.returnValue(of({
      id: '123',
      question: 'Who knows better how to cook pasta?',
      groomScore: 5,
      brideScore: 10
    })),
    submitAnswer: jasmine.createSpy('submitAnswer').and.returnValue(of({}))
  };

  const mockActivatedRoute = {
    snapshot: {
      paramMap: {
        get: (key: string) => {
          if (key === 'weddingId') return '123';
          if (key === 'guestId') return '456';
          return null;
        }
      }
    }
  };

  const mockRouter = {
    navigateByUrl: jasmine.createSpy('navigateByUrl')
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [WhoKnowsBetterQuizComponent],
      providers: [
        {provide: WhoKnowsBetterQuizService, useValue: mockWhoKnowsBetterQuizService},
        {provide: ActivatedRoute, useValue: mockActivatedRoute},
        {provide: Router, useValue: mockRouter}
      ]
    })
    .compileComponents();

    const fixture = TestBed.createComponent(WhoKnowsBetterQuizComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should initialize weddingId and guestId in ngOnInit', () => {
    component.ngOnInit();
    expect(component.weddingId).toEqual('123');
    expect(component.guestId).toEqual('456');
  });

  it('should load quiestion if quiz is not finished',()=>{
    component.ngOnInit();
    expect(mockWhoKnowsBetterQuizService.getWhoKnowsBetterQuizStateByWeddingId).toHaveBeenCalledWith('123','456');
    expect(mockWhoKnowsBetterQuizService.getQuestion).toHaveBeenCalledWith('123','456');
    expect(component.currentQuestion.question).toBe('Who knows better how to cook pasta?');
    expect(component.questionAnswered).toBeFalse();
    expect(component.selectedAnswer).toBeNull();
  });

  it('should show summary when quiz is finished', () => {
    mockWhoKnowsBetterQuizService.getWhoKnowsBetterQuizStateByWeddingId.and.returnValue(
      of({ whoKnowsQuizFinished: true })
    );

    spyOn(component, 'toggleSummary').and.callThrough();
    component.ngOnInit();

    expect(mockWhoKnowsBetterQuizService.getWhoKnowsBetterQuizStateByWeddingId).toHaveBeenCalledWith('123', '456');
    expect(mockWhoKnowsBetterQuizService.getQuestion).not.toHaveBeenCalled();
    expect(component.toggleSummary).toHaveBeenCalled();
    expect(component.isSummaryReady).toBeTrue();
  });

  it('should submit groom when selected', () => {
    component.ngOnInit();
    component.submitAnswer('groom');

    expect(component.selectedAnswer).toBe('groom');
    expect(component.questionAnswered).toBeTrue();
    expect(mockWhoKnowsBetterQuizService.submitAnswer).toHaveBeenCalledWith('123', '456', 'groom');
  });

  it('should not submit answer if question already answered', () => {
    component.ngOnInit();
    component.questionAnswered = true;
    component.submitAnswer('groom');

    expect(mockWhoKnowsBetterQuizService.submitAnswer).not.toHaveBeenCalled();
  });

  it('should set bride when selected', () => {
    component.ngOnInit();
    component.submitAnswer('bride');

    expect(component.selectedAnswer).toBe('bride');
    expect(mockWhoKnowsBetterQuizService.submitAnswer).toHaveBeenCalledWith('123', '456', 'bride');
  });

});
