import {Component, OnInit} from '@angular/core';
import { MessageService, PrimeNGConfig} from 'primeng/api';
import {FloatLabelModule} from 'primeng/floatlabel';
import {FormsModule} from '@angular/forms';
import {InputTextareaModule} from 'primeng/inputtextarea';
import {ToastModule} from 'primeng/toast';
import {FileUploadModule} from 'primeng/fileupload';
import {BadgeModule} from 'primeng/badge';
import {NgClass, NgForOf, NgIf} from '@angular/common';
import {CalendarModule} from 'primeng/calendar';
import {ChipsModule} from 'primeng/chips';
import {CheckboxModule} from 'primeng/checkbox';
import {ListboxModule} from 'primeng/listbox';

interface Game {
  name: string;
  code: string;
}

@Component({
  selector: 'app-event-page-configuration',
  standalone: true,
  imports: [
    FloatLabelModule,
    FormsModule,
    InputTextareaModule,
    ToastModule,
    FileUploadModule,
    BadgeModule,
    NgClass,
    NgIf,
    NgForOf,
    CalendarModule,
    ChipsModule,
    CheckboxModule,
    ListboxModule
  ],
  templateUrl: './event-page-configuration.component.html',
  styleUrl: './event-page-configuration.component.css',
  host: { ngSkipHydration: ''}
})
export class EventPageConfigurationComponent implements OnInit {

  eventName: string | undefined;

  eventDescription: string | undefined;

  datetime24h: Date | undefined;

  files: File[] = [];

  totalSize : number = 0;

  totalSizePercent : number = 0;

  selectedGames!: Game[];

  games!: Game[];

  constructor(private config: PrimeNGConfig, private messageService: MessageService) {}

  ngOnInit() {
    this.games = [
      { name: 'Quizz', code: 'Q' },
      { name: 'Kto bardziej?', code: 'W' }
    ]
  }

  choose(event: any, callback: () => void) {
    callback();
  }

  onRemoveTemplatingFile(event: any, file: { size: any; }, removeFileCallback: (arg0: any, arg1: any) => void, index: any) {
    removeFileCallback(event, index);
    this.totalSize -= parseInt(this.formatSize(file.size));
    this.totalSizePercent = this.totalSize / 10;
  }

  onClearTemplatingUpload(clear: () => void) {
    clear();
    this.totalSize = 0;
    this.totalSizePercent = 0;
  }

  onTemplatedUpload() {
    this.messageService.add({ severity: 'info', summary: 'Success', detail: 'File Uploaded', life: 3000 });
  }

  onSelectedFiles(event: any) {
    this.files = event.currentFiles;
    this.files.forEach((file) => {
      this.totalSize += parseInt(this.formatSize(file.size));
    });
    this.totalSizePercent = this.totalSize / 10;
  }

  uploadEvent(callback: () => void) {
    callback();
  }

  formatSize(bytes: number) {
    const k = 1024;
    const dm = 3;
    const sizes = this.config.translation.fileSizeTypes;
    if (bytes === 0) {
      return `0 ${sizes ? sizes : [0]}`;
    }

    const i = Math.floor(Math.log(bytes) / Math.log(k));
    const formattedSize = parseFloat((bytes / Math.pow(k, i)).toFixed(dm));

    return `${formattedSize} ${sizes ? sizes : [i]}`;
  }

  protected readonly String = String;
}
