import { ComponentFixture, TestBed } from '@angular/core/testing';

import { GuestsComponent } from './guests.component';
import {GuestsService} from '../../../services/guests.service';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {provideRouter} from '@angular/router';
import {provideHttpClient} from '@angular/common/http';
import {provideHttpClientTesting} from '@angular/common/http/testing';
import {GuestType} from '../../../models/guests.type';
import {ConfirmationGroupType} from '../../../models/confirmationGroup.type';
import {of} from 'rxjs';

describe('GuestsComponent', () => {
  let component: GuestsComponent;
  let fixture: ComponentFixture<GuestsComponent>;
  let native: HTMLElement;
  let guestService: jasmine.SpyObj<GuestsService>;

  beforeEach(async () => {
    const spy = jasmine.createSpyObj<GuestsService>('GuestService', ['getGuestsByWeddingId','patchGuestInformation','postGuestInformation','deleteGuestInformation']);

    await TestBed.configureTestingModule({
      imports: [
        GuestsComponent,
        NoopAnimationsModule
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: GuestsService,
          useValue: spy
        }
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(GuestsComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    guestService = TestBed.inject(GuestsService) as jasmine.SpyObj<GuestsService>;

    const confirmationGroup: ConfirmationGroupType ={
      id: '99921198-9a7a-425a-a211-989a7a525a23',
      name: 'Bracia'
    }

    const guests: [GuestType] = [
      {
        id: '84621198-9a7a-425a-a211-989a7a525a23',
        presenceConfirmed: null,
        guestName: 'ARO',
        guestAge: 'Dorosły',
        weddingConfirmationGroup:confirmationGroup
      }
    ];
    guestService.getGuestsByWeddingId.and.returnValue(of(guests));

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render table with guests', () => {
    // TODO: poprawic testy
    // component.weddingGuests = [{
    //   id: '11121198-9a7a-425a-a211-989a7a525a23',
    //   guestName: 'Jan',
    //   guestAge: 'Dorosły',
    //   presenceConfirmed: true,
    //   weddingConfirmationGroup: {
    //     id: '11121198-9a7a-425a-a211-989a7a525222',
    //     name: 'Rodzina'
    //   }
    // }];
    fixture.detectChanges(); //will have to look into that

    let containers = native.querySelectorAll('.flex.flex-wrap.align-items-center');
    expect(containers.length).toBe(1);
  });

  it('should call saveGuest when save button is clicked', () => {
    spyOn(component, 'saveGuest');
    component.guestDialog = true;
    fixture.detectChanges();

    const saveButton = native.querySelectorAll('p-button[icon="pi pi-check"]');
    saveButton.item(0).dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.saveGuest).toHaveBeenCalled();
  });

  it('should call deleteGuest when delete button is clicked', () => {
    spyOn(component, 'deleteGuest');
    let guest:GuestType = {
      id: '11121198-9a7a-425a-a211-989a7a525a23',
      guestName: 'Jan',
      guestAge: 'Dorosły',
      presenceConfirmed: true,
      weddingConfirmationGroup: {
        id: '11121198-9a7a-425a-a211-989a7a525222',
        name: 'Rodzina'
      }
    }
    // component.weddingGuests = [guest];
    fixture.detectChanges();

    const rowTogglerButton = native.querySelector('p-table tbody tr .p-button-text');
    rowTogglerButton!.dispatchEvent(new Event('click'));
    fixture.detectChanges();

    const rowExpansion = native.querySelector('p-table tbody tr.p-row-expansion');
    const deleteButton = rowExpansion!.querySelector('p-button[icon="pi pi-trash"]');
    deleteButton!.dispatchEvent(new Event('click'));
    fixture.detectChanges();

    // expect(component.deleteGuest).toHaveBeenCalledWith(guest);
  });

  it('should show edit/add dialog for guest when add/edit guest button is clicked', () => {
    fixture.detectChanges();
    let addButton= native.querySelector('.add-image-container p-button button');

    addButton!.dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.guestDialog).toBe(true);
  });

});
