import { ComponentFixture, TestBed } from '@angular/core/testing';

import { PresentUserComponent } from './present-user.component';
import {PresentService} from '../../../services/present.service';
import {provideRouter} from '@angular/router';
import {provideHttpClient} from '@angular/common/http';
import {provideHttpClientTesting} from '@angular/common/http/testing';
import {PresentType} from '../../../models/presents.type';
import {of} from 'rxjs';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {Confirmation, ConfirmationService} from 'primeng/api';

describe(PresentUserComponent.name, () => {
  let component: PresentUserComponent;
  let fixture: ComponentFixture<PresentUserComponent>;
  let native:HTMLElement;
  let presentService:jasmine.SpyObj<PresentService>;

  beforeEach(async () => {
    const spy = jasmine.createSpyObj<PresentService>('PresentService',['getPresentsByWeddingId','patchPresentDataModifiableByUser','postPresentToCreate','deletePresentByUser']);

    await TestBed.configureTestingModule({
      imports: [
        PresentUserComponent,
        NoopAnimationsModule
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: PresentService,
          useValue: spy
        }
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(PresentUserComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    presentService = TestBed.inject(PresentService) as jasmine.SpyObj<PresentService>;

    const present: [PresentType]=[
      {
        id: '84621198-9a7a-425a-a211-989a7a525a23',
        name: 'iksboks',
        linkToPresent: 'best/site/with/iksbokses',
        description: 'iksbox 360',
        reservedBy: '84621198-9a7a-425a-a211-989a7a525a21',
      }
    ]
    presentService.getPresentsByWeddingId.and.returnValue(of(present));

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render presents view', () => {
    let containers = native.querySelectorAll('.card');
    expect(containers.length).toBe(1);
  });

  it('should show add/edit dialog for add/edit present when add/edit button is clicked', () => {
    fixture.detectChanges();
    let addButton= native.querySelector('.card p-button');

    addButton!.dispatchEvent(new Event('click'));
    fixture.detectChanges();

    expect(component.presentDialog).toBe(true);
  });

  it('should send guest when savePresent is called with new present', () => {
    presentService.postPresentToCreate.and.returnValue(of(null));

    component.presentForm.name = 'a';
    fixture.detectChanges();

    component.savePresent();

    expect(presentService.postPresentToCreate).toHaveBeenCalledWith(component.presentForm);
  });

  it('should call deletePresent when delete button is clicked', () => {
    presentService.deletePresentByUser.and.returnValue(of(null));

    spyOn(component['confirmationService'], 'confirm').and.callFake((confirmation: Confirmation) => { return confirmation.accept?.(); });

    component.presentForm.id = 'cos';
    component.presentForm.name = 'New Present';
    component.presentForm.linkToPresent = 'new/link';
    component.presentForm.description = 'New present description';
    component.presentForm.reservedBy = 'some-user-id';

    component.deletePresent(component.presentForm);

    expect(presentService.deletePresentByUser).toHaveBeenCalledWith(component.presentForm);
  });

  it('should send present when savePresent is called with present that exists', () => {
    presentService.patchPresentDataModifiableByUser.and.returnValue(of(null));

    component.presentForm.name = 'a';
    component.presentForm.id='niezero'
    fixture.detectChanges();

    component.savePresent();

    expect(presentService.patchPresentDataModifiableByUser).toHaveBeenCalledWith(component.presentForm);
  });

});
