import { Pipe, PipeTransform } from '@angular/core';

@Pipe({
  name: 'filter',
  standalone: true
})
export class FilterPipe implements PipeTransform {

  transform(value: any[], searchText: string, fields: string[]): any[] {
    if (!value || !searchText) return value;

    searchText = searchText.toLowerCase();

    return value
      .map(group => {
        const groupMatches = fields.includes('name') &&
          group.name?.toString().toLowerCase().includes(searchText);

        const guestFields = fields.filter(f => f.startsWith('guestList.'));
        let filteredGuests: any[] = [];

        if (guestFields.length) {
          filteredGuests = group.guestList.filter((guest: any) =>
            guestFields.some(field => {
              const guestField = field.split('.').slice(1).join('.');
              const val = this.getNestedValue(guest, guestField);
              return !!(val && val.toString().toLowerCase().includes(searchText));
            })
          );
        }

        if (groupMatches) {
          return group;
        }

        if (filteredGuests.length > 0) {
          return {
            ...group,
            guestList: filteredGuests
          };
        }

        return null;
      })
      .filter(group => group !== null);
  }

  private getNestedValue(obj: any, path: string): any {
    return path.split('.').reduce((acc, key) => acc && acc[key], obj);
  }

}
