import {inject, Injectable} from '@angular/core';
import {HttpClient, HttpHeaders, HttpParams} from '@angular/common/http';
import {Observable} from 'rxjs';
import {QuizType} from '../models/quiz.type';
import {QuizStateType} from '../models/quizState.type';
import {QuizScoreType} from '../models/quizScore.type';
import {GuestType} from '../models/guests.type';
import {KeycloakService} from '../keycloak/keycloak.service';
import {ConfirmationGroupType, ConfirmationGroupWithGuests} from '../models/confirmationGroup.type';


@Injectable({
  providedIn: 'root'
})
export class GuestsService {

  constructor(private http:HttpClient,private keycloakService: KeycloakService) {}

  getGuestsByWeddingId(): Observable<[GuestType]> {
    return this.http.get<[GuestType]>(`/api/users/guests`);
  }

  getGroupsWithGuestsByWeddingId(): Observable<[ConfirmationGroupWithGuests]> {
    return this.http.get<[ConfirmationGroupWithGuests]>(`/api/users/groups/guests`);
  }

  patchCharacterPresenceConfirmation(weddingId: string, guestId: string, presenceConfirmed: boolean): Observable<any> {
    let params = new HttpParams().set('presenceConfirmed', presenceConfirmed);

    return this.http.patch(`/api/wedding/${weddingId}/guests/${guestId}`, null,  { params });
  }

  patchGuestInformation(guest:GuestType): Observable<any> {
    const body= { ...guest };
    return this.http.patch(`api/users/guests`, body);
  }

  postGuestInformation(guest:GuestType): Observable<any> {
    const body= { ...guest };
    return this.http.post(`api/users/guests`, body);
  }

  deleteGuestInformation(guest:GuestType): Observable<any> {
    const body = { ...guest };
    return this.http.delete(`api/users/guests`, {body});
  }

  //groups
  deleteGroup(group:ConfirmationGroupType): Observable<any> {
    const body= { ...group };
    return this.http.delete(`api/users/groups/guests`, {body});
  }

  createGroup(group:ConfirmationGroupType): Observable<any> {
    const body= { ...group };
    return this.http.post(`api/users/groups/guests`, body);
  }

}
