import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from '@angular/common/http';
import {KeycloakService} from '../keycloak/keycloak.service';
import {Observable} from 'rxjs';
import {PresentType} from '../models/presents.type';
import {GuestType} from '../models/guests.type';

@Injectable({
  providedIn: 'root'
})
export class PresentService {

  constructor(private http:HttpClient,private keycloakService: KeycloakService) { }

  getPresentsByWeddingId(): Observable<[PresentType]> {
    return this.http.get<[PresentType]>(`api/users/presents`);
  }

  patchPresentDataModifiableByUser(present:PresentType): Observable<any> {

    const body= { ...present };
    return this.http.patch(`api/users/presents`, body);
  }

  postPresentToCreate(present:PresentType): Observable<any> {

    const body= { ...present };
    return this.http.post(`api/users/presents`, body);
  }

  deletePresentByUser(present:PresentType): Observable<any> {

    const body = { ...present };
    return this.http.delete(`api/users/presents`, {body });
  }

  //czesc gosci
  getPresentsByWeddingIdGuestSide(weddingId: string,guestId:string): Observable<[PresentType]> {
    return this.http.get<[PresentType]>(`api/weddings/${weddingId}/${guestId}/presents`);
  }

  sendPresentsToEditByWeddingId(weddingId: string,guestId:string,present:PresentType): Observable<any> {
    const body = { ...present };
    return this.http.patch(`api/weddings/${weddingId}/${guestId}/presents`, body);
  }

}
